import { onUnmounted, watch, ref, inject, onMounted, reactive, provide, onBeforeUnmount, computed, openBlock, createBlock, renderSlot, createCommentVNode, h } from 'vue';

const props = {
  position: {
    type: String,
    default: "topright",
  },
};
const setup = (mapRef) => {
  onUnmounted(() => {
    if (mapRef.value) {
      mapRef.value.remove();
    }
  });
};

const props$1 = {
  ...props,
  collapsed: {
    type: Boolean,
    default: true,
  },
  autoZIndex: {
    type: Boolean,
    default: true,
  },
  hideSingleBase: {
    type: Boolean,
    default: false,
  },
  sortLayers: {
    type: Boolean,
    default: false,
  },
  sortFunction: {
    type: Function,
    default: undefined,
  },
};

const setup$1 = (props, mapRef) => {
  setup(mapRef);
  const options = {
    collapsed: props.collapsed,
    autoZIndex: props.autoZIndex,
    hideSingleBase: props.hideSingleBase,
    sortLayers: props.sortLayers,
    sortFunction: props.sortFunction,
  };

  const methods = {
    addLayer(layer) {
      if (layer.layerType === "base") {
        mapRef.value.addBaseLayer(layer.mapObject, layer.name);
      } else if (layer.layerType === "overlay") {
        mapRef.value.addOverlay(layer.mapObject, layer.name);
      }
    },
    removeLayer(layer) {
      mapRef.value.removeLayer(layer.mapObject);
    },
  };
  return { options, methods };
};

const debounce = (fn, time) => {
  let timeout;

  return function (...args) {
    const context = this;
    if (timeout) {
      clearTimeout(timeout);
    }
    timeout = setTimeout(() => {
      fn.apply(context, args);
      timeout = null;
    }, time);
  };
};

const capitalizeFirstLetter = (string) => {
  if (!string || typeof string.charAt !== "function") {
    return string;
  }
  return string.charAt(0).toUpperCase() + string.slice(1);
};

const propsBinder = (methods, leafletElement, props, setOptions) => {
  for (const key in props) {
    const setMethodName = "set" + capitalizeFirstLetter(key);
    if (methods[setMethodName]) {
      watch(
        () => props[key],
        (newVal, oldVal) => {
          methods[setMethodName](newVal, oldVal);
        }
      );
    } else if (setMethodName === "setOptions") {
      watch(
        () => props[key],
        (newVal) => {
          setOptions(leafletElement, newVal);
        }
      );
    } else if (leafletElement[setMethodName]) {
      watch(
        () => props[key],
        (newVal) => {
          leafletElement[setMethodName](newVal);
        }
      );
    }
  }
};

const remapEvents = (onEvent) => {
  const result = {};
  for (const eventName in onEvent) {
    if (eventName !== "modelValue" && !eventName.startsWith("onUpdate")) {
      const newName = eventName.replace("on", "").toLowerCase();
      result[newName] = onEvent[eventName];
    }
  }
  return result;
};

const resetWebpackIcon = (Icon) => {
  delete Icon.Default.prototype._getIconUrl;

  Icon.Default.mergeOptions({
    iconRetinaUrl: require("leaflet/dist/images/marker-icon-2x.png"),
    iconUrl: require("leaflet/dist/images/marker-icon.png"),
    shadowUrl: require("leaflet/dist/images/marker-shadow.png"),
  });
};

var script = {
  name: "LControlLayers",
  props: props$1,
  setup(props) {
    const mapRef = ref({});

    const lMethods = inject("leafLetMethods");
    const { options, methods } = setup$1(props, mapRef);
    onMounted(async () => {
      const { control, setOptions } = await import(
        'leaflet/dist/leaflet-src.esm'
      );

      mapRef.value = control.layers(null, null, options);
      propsBinder(methods, mapRef.value, props, setOptions);

      lMethods.registerLayerControl({
        ...props,
        ...methods,
        mapObject: mapRef.value,
      });
    });
  },
  render() {
    return null;
  },
};

const render = () => {};


script.render = render;
script.__file = "src/components/LControlLayers.vue";

var script$1 = {
  props: {
    /**
     * The center of the map, supports .sync modifier
     */
    center: {
      type: [Object, Array],
      default: () => [0, 0],
    },
    /**
     * The bounds of the map, supports .sync modifier
     */
    bounds: {
      type: [Array, Object],
      default: undefined,
    },
    /**
     * The max bounds of the map
     */
    maxBounds: {
      type: [Array, Object],
      default: undefined,
    },
    /**
     * The zoom of the map, supports .sync modifier
     */
    zoom: {
      type: Number,
      default: 0,
    },
    /**
     * The minZoom of the map
     */
    minZoom: {
      type: Number,
      default: undefined,
    },
    /**
     * The maxZoom of the map
     */
    maxZoom: {
      type: Number,
      default: undefined,
    },
    /**
     * The paddingBottomRight of the map
     */
    paddingBottomRight: {
      type: Array,
      default: undefined,
    },
    /**
     * The paddingTopLeft of the map
     */
    paddingTopLeft: {
      type: Array,
      default: undefined,
    },
    /**
     * The padding of the map
     */
    padding: {
      type: Array,
      default: undefined,
    },
    /**
     * The worldCopyJump option for the map
     */
    worldCopyJump: {
      type: Boolean,
      default: false,
    },
    /**
     * The crs option for the map
     * @values CRS.EPSG3857
     */
    crs: {
      type: Object,
    },
    maxBoundsViscosity: {
      type: Number,
      default: undefined,
    },
    inertia: {
      type: Boolean,
      default: undefined,
    },
    inertiaDeceleration: {
      type: Number,
      default: undefined,
    },
    inertiaMaxSpeed: {
      type: Number,
      default: undefined,
    },
    easeLinearity: {
      type: Number,
      default: undefined,
    },
    zoomAnimation: {
      type: Boolean,
      default: undefined,
    },
    zoomAnimationThreshold: {
      type: Number,
      default: undefined,
    },
    fadeAnimation: {
      type: Boolean,
      default: undefined,
    },
    markerZoomAnimation: {
      type: Boolean,
      default: undefined,
    },
    noBlockingAnimations: {
      type: Boolean,
      default: false,
    },
  },
  setup(props, context) {
    const root = ref(null);
    const blueprint = reactive({
      ready: false,
      mapRef: {},
      layersToAdd: [],
      layersInControl: [],
    });

    const options = {
      minZoom: props.minZoom,
      maxZoom: props.maxZoom,
      maxBounds: props.maxBounds,
      maxBoundsViscosity: props.maxBoundsViscosity,
      worldCopyJump: props.worldCopyJump,
      crs: props.crs,
      center: props.center,
      zoom: props.zoom,
      inertia: props.inertia,
      inertiaDeceleration: props.inertiaDeceleration,
      inertiaMaxSpeed: props.inertiaMaxSpeed,
      easeLinearity: props.easeLinearity,
      zoomAnimation: props.zoomAnimation,
      zoomAnimationThreshold: props.zoomAnimationThreshold,
      fadeAnimation: props.fadeAnimation,
      markerZoomAnimation: props.markerZoomAnimation,
    };

    const schematics = reactive({
      addLayer() {},
      removeLayer() {},
      registerLayerControl() {},
    });

    provide("leafLetMethods", schematics);

    const eventHandlers = {
      moveEndHandler() {
        /**
         * Triggers when zoom is updated
         * @type {number,string}
         */
        context.emit("update:zoom", blueprint.mapRef.getZoom());
        /**
         * Triggers when center is updated
         * @type {object,array}
         */
        context.emit("update:center", blueprint.mapRef.getCenter());

        /**
         * Triggers when bounds are updated
         * @type {object}
         */
        context.emit("update:bounds", blueprint.mapRef.getBounds());
      },
      overlayAddHandler(e) {
        const layer = blueprint.layersInControl.find((l) => l.name === e.name);
        if (layer) {
          layer.updateVisibleProp(true);
        }
      },
      overlayRemoveHandler(e) {
        const layer = blueprint.layersInControl.find((l) => l.name === e.name);
        if (layer) {
          layer.updateVisibleProp(false);
        }
      },
    };

    onMounted(async () => {
      const {
        map,
        CRS,
        Icon,
        latLngBounds,
        latLng,
        DomEvent,
        setOptions,
      } = await import('leaflet/dist/leaflet-src.esm');
      resetWebpackIcon(Icon);
      options.crs = options.crs || CRS.EPSG3857;

      const methods = {
        addLayer(layer) {
          if (layer.layerType !== undefined) {
            if (blueprint.layerControl === undefined) {
              blueprint.layersToAdd.push(layer);
            } else {
              const exist = blueprint.layersInControl.find(
                (l) => l.mapObject._leaflet_id === layer.mapObject._leaflet_id
              );
              if (!exist) {
                blueprint.layerControl.addLayer(layer);
                blueprint.layersInControl.push(layer);
              }
            }
          }
          if (layer.visible !== false) {
            blueprint.mapRef.addLayer(layer.mapObject);
          }
        },
        removeLayer(layer) {
          if (layer.layerType !== undefined) {
            if (blueprint.layerControl === undefined) {
              blueprint.layersToAdd = blueprint.layersToAdd.filter(
                (l) => l.name !== layer.name
              );
            } else {
              blueprint.layerControl.removeLayer(layer.mapObject);
              blueprint.layersInControl = blueprint.layersInControl.filter(
                (l) => l.mapObject._leaflet_id !== layer.mapObject._leaflet_id
              );
            }
          }
          blueprint.mapRef.removeLayer(layer.mapObject);
        },

        registerLayerControl(lControlLayer) {
          blueprint.layerControl = lControlLayer;
          blueprint.mapRef.addControl(lControlLayer.mapObject);
          blueprint.layersToAdd.forEach((layer) => {
            blueprint.layerControl.addLayer(layer);
          });
          blueprint.layersToAdd = [];
        },

        setZoom(newVal) {
          blueprint.mapRef.setZoom(newVal, {
            animate: props.noBlockingAnimations ? false : null,
          });
        },

        setPaddingBottomRight(newVal) {
          blueprint.paddingBottomRight = newVal;
        },
        setPaddingTopLeft(newVal) {
          blueprint.paddingTopLeft = newVal;
        },
        setPadding(newVal) {
          blueprint.padding = newVal;
        },
        setCrs(newVal) {
          const prevBounds = blueprint.mapRef.getBounds();
          blueprint.mapRef.options.crs = newVal;
          blueprint.mapRef.fitBounds(prevBounds, {
            animate: false,
            padding: [0, 0],
          });
        },
        fitBounds(bounds) {
          blueprint.mapRef.fitBounds(bounds, {
            animate: this.noBlockingAnimations ? false : null,
          });
        },
        setBounds(newVal) {
          if (!newVal) {
            return;
          }
          const newBounds = latLngBounds(newVal);
          if (!newBounds.isValid()) {
            return;
          }
          const oldBounds =
            blueprint.lastSetBounds || blueprint.mapRef.getBounds();
          const boundsChanged = !oldBounds.equals(newBounds, 0); // set maxMargin to 0 - check exact equals
          if (boundsChanged) {
            blueprint.lastSetBounds = newBounds;
            blueprint.mapRef.fitBounds(newBounds, this.fitBoundsOptions);
          }
        },

        setCenter(newVal) {
          if (newVal == null) {
            return;
          }
          const newCenter = latLng(newVal);
          const oldCenter =
            blueprint.lastSetCenter || blueprint.mapRef.getCenter();
          if (
            oldCenter.lat !== newCenter.lat ||
            oldCenter.lng !== newCenter.lng
          ) {
            blueprint.lastSetCenter = newCenter;
            blueprint.mapRef.panTo(newCenter, {
              animate: this.noBlockingAnimations ? false : null,
            });
          }
        },
      };

      schematics.addLayer = methods.addLayer;
      schematics.removeLayer = methods.removeLayer;
      schematics.registerLayerControl = methods.registerLayerControl;

      blueprint.mapRef = map(root.value, options);

      propsBinder(methods, blueprint.mapRef, props, setOptions);
      const listeners = remapEvents(context.attrs);

      blueprint.mapRef.on(
        "moveend",
        debounce(eventHandlers.moveEndHandler, 100)
      );
      blueprint.mapRef.on("overlayadd", eventHandlers.overlayAddHandler);
      blueprint.mapRef.on("overlayremove", eventHandlers.overlayRemoveHandler);
      DomEvent.on(blueprint.mapRef, listeners);
      blueprint.ready = true;
    });

    onBeforeUnmount(() => {
      if (blueprint.mapRef) {
        blueprint.mapRef.remove();
      }
    });

    const mapObject = computed(() => blueprint.mapRef);
    const ready = computed(() => blueprint.ready);
    return { root, ready, mapObject };
  },
};

const _hoisted_1 = {
  style: {"width":"100%","height":"100%"},
  ref: "root"
};

function render$1(_ctx, _cache, $props, $setup, $data, $options) {
  return (openBlock(), createBlock("div", _hoisted_1, [
    ($setup.ready)
      ? renderSlot(_ctx.$slots, "default", { key: 0 })
      : createCommentVNode("v-if", true)
  ], 512 /* NEED_PATCH */))
}

script$1.render = render$1;
script$1.__file = "src/components/LMap.vue";

const props$2 = {
  pane: {
    type: String,
    default: "overlayPane",
  },
  attribution: {
    type: String,
    default: null,
  },
  name: {
    type: String,
    custom: true,
    default: undefined,
  },
  layerType: {
    type: String,
    custom: true,
    default: undefined,
  },
  visible: {
    type: Boolean,
    custom: true,
    default: true,
  },
};

const setup$2 = (props, mapRef, context) => {
  const lMethods = inject("leafLetMethods");
  const options = {
    attribution: props.attribution,
    pane: props.pane,
  };

  const methods = {
    setAttribution(val, old) {
      const attributionControl = this.$parent.mapObject.attributionControl;
      attributionControl.removeAttribution(old).addAttribution(val);
    },
    setName() {
      lMethods.removeLayer(mapRef.value);
      if (props.visible) {
        lMethods.addLayer(mapRef.value);
      }
    },
    setLayerType() {
      lMethods.removeLayer(mapRef.value);
      if (props.visible) {
        lMethods.addLayer(mapRef.value);
      }
    },
    setVisible(isVisible) {
      if (mapRef.value) {
        if (isVisible) {
          lMethods.addLayer(mapRef.value);
        } else {
          lMethods.removeLayer(mapRef.value);
        }
      }
    },
    bindTooltip({ mapObject }) {
      mapRef.value.bindTooltip(mapObject);
    },
    unbindTooltip() {
      const tooltip = mapRef.value ? mapRef.value.getTooltip() : null;
      if (tooltip) {
        tooltip.unbindTooltip();
      }
    },
    unbindPopup() {
      const popup = mapRef.value ? mapRef.value.getPopup() : null;
      if (popup) {
        popup.unbindPopup();
      }
    },
    updateVisibleProp(value) {
      /**
       * Triggers when the visible prop needs to be updated
       * @type {boolean}
       * @property {boolean} value - value of the visible property
       */
      context.emit("update:visible", value);
    },
  };

  provide("leafLetMethods", {
    ...lMethods,
    bindTooltip: methods.bindTooltip,
    unbindTooltip: methods.unbindTooltip,
  });

  onUnmounted(() => {
    methods.unbindPopup();
    methods.unbindTooltip();
    lMethods.removeLayer({ mapObject: mapRef.value });
  });

  return { options, methods };
};

const props$3 = {
  ...props$2,
  pane: {
    type: String,
    default: "markerPane",
  },
  draggable: {
    type: Boolean,
    custom: true,
    default: false,
  },
  latLng: {
    type: [Object, Array],
    custom: true,
    default: null,
  },
  icon: {
    type: [Object],
    custom: false,
  },
  zIndexOffset: {
    type: Number,
    custom: false,
    default: null,
  },
};

const setup$3 = (props, mapRef, context, leafletMethods) => {
  const { options: layerOptions, methods: layerMethods } = setup$2(
    props,
    mapRef,
    context
  );
  const options = {
    ...layerOptions,
    ...props,
  };

  const methods = {
    ...layerMethods,
    setDraggable(value) {
      if (mapRef.value.dragging) {
        value
          ? mapRef.value.dragging.enable()
          : mapRef.value.dragging.disable();
      }
    },
    latLngSync(event) {
      context.emit("update:latLng", event.latlng);
      context.emit("update:lat-lng", event.latlng);
    },
    setLatLng(newVal) {
      if (newVal == null) {
        return;
      }

      if (mapRef.value) {
        const oldLatLng = mapRef.value.getLatLng();
        const newLatLng = leafletMethods.latLng(newVal);
        if (
          newLatLng.lat !== oldLatLng.lat ||
          newLatLng.lng !== oldLatLng.lng
        ) {
          mapRef.value.setLatLng(newLatLng);
        }
      }
    },
  };
  return { options, methods };
};

/**
 * Marker component, lets you add and personalize markers on the map
 */
var script$2 = {
  name: "LMarker",
  props: props$3,
  setup(props, context) {
    const mapRef = ref({});
    const ready = ref(false);

    const schematics = reactive({
      latLng() {},
    });

    const lMethods = inject("leafLetMethods");
    const { options, methods } = setup$3(
      props,
      mapRef,
      context,
      schematics
    );

    onMounted(async () => {
      const { marker, DomEvent, latLng, setOptions } = await import(
        'leaflet/dist/leaflet-src.esm'
      );
      schematics.latLng = latLng;

      mapRef.value = marker(props.latLng, options);

      const listeners = remapEvents(context.attrs);
      DomEvent.on(mapRef.value, listeners);

      mapRef.value.on("move", debounce(methods.latLngSync, 100));
      propsBinder(methods, mapRef.value, props, setOptions);
      lMethods.addLayer({ ...props, ...methods, mapObject: mapRef.value });
      ready.value = true;
    });
    return { ready };
  },
  render() {
    if (this.ready && this.$slots.default) {
      return h("div", { style: { display: "none" } }, this.$slots.default());
    }
    return null;
  },
};

const render$2 = () => {};


script$2.render = render$2;
script$2.__file = "src/components/LMarker.vue";

const props$4 = {
  ...props$2,
  pane: {
    type: String,
    default: "tilePane",
  },
  opacity: {
    type: Number,
    custom: false,
    default: 1.0,
  },
  zIndex: {
    type: Number,
    default: 1,
  },
  tileSize: {
    type: Number,
    default: 256,
  },
  noWrap: {
    type: Boolean,
    default: false,
  },
};

const setup$4 = (props, mapRef) => {
  const { options: layerOptions, methods: layerMethods } = setup$2(
    props,
    mapRef
  );
  const options = {
    ...layerOptions,
    pane: props.pane,
    opacity: props.opacity,
    zIndex: props.zIndex,
    tileSize: props.tileSize,
    noWrap: props.noWrap,
  };
  return { options, methods: { ...layerMethods } };
};

const props$5 = {
  ...props$4,
  tms: {
    type: Boolean,
    default: false,
  },
  subdomains: {
    type: String,
    default: "abc",
  },
  detectRetina: {
    type: Boolean,
    default: false,
  },
  url: {
    type: String,
    default: null,
  },
};

const setup$5 = (props, mapRef) => {
  const {
    options: gridLayerOptions,
    methods: gridLayerMethods,
  } = setup$4(props, mapRef);
  const options = {
    ...gridLayerOptions,
    tms: props.tms,
    subdomains: props.subdomains,
    detectRetina: props.detectRetina,
  };
  return {
    options,
    methods: {
      ...gridLayerMethods,
    },
  };
};

var script$3 = {
  props: props$5,
  setup(props, context) {
    const mapRef = ref({});
    const lMethods = inject("leafLetMethods");

    const { options, methods } = setup$5(props, mapRef);

    onMounted(async () => {
      const { tileLayer, DomEvent, setOptions } = await import(
        'leaflet/dist/leaflet-src.esm'
      );
      mapRef.value = tileLayer(props.url, options);

      const listeners = remapEvents(context.attrs);
      DomEvent.on(mapRef.value, listeners);

      propsBinder(methods, mapRef.value, props, setOptions);
      lMethods.addLayer({ ...props, ...methods, mapObject: mapRef.value });
    });
  },
  render() {
    return null;
  },
};

const render$3 = () => {};


script$3.render = render$3;
script$3.__file = "src/components/LTileLayer.vue";

export { script as LControlLayers, script$1 as LMap, script$2 as LMarker, script$3 as LTileLayer };
//# sourceMappingURL=vue-leaflet.esm.js.map
